﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Input.StylusPlugIns;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 커스텀 동적 렌더러
    /// </summary>
    public class CustomDynamicRenderer : DynamicRenderer
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 브러시
        /// </summary>
        [ThreadStatic]
        private static Brush _brush = null;

        /// <summary>
        /// 펜
        /// </summary>
        [ThreadStatic]
        private static Pen _pen = null;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 이전 포인트
        /// </summary>
        private Point previousPoint;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Method
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 스타일러스 DOWN 처리하기 - OnStylusDown(rawStylusInput)

        /// <summary>
        /// 스타일러스 DOWN 처리하기
        /// </summary>
        /// <param name="rawStylusInput">원시 스타일러스 입력</param>
        protected override void OnStylusDown(RawStylusInput rawStylusInput)
        {
            this.previousPoint = new Point(double.NegativeInfinity, double.NegativeInfinity);

            base.OnStylusDown(rawStylusInput);
        }

        #endregion
        #region 그리기 처리하기 - OnDraw(drawingContext, stylusPointCollection, geometry, fillBrush)

        /// <summary>
        /// 그리기 처리하기
        /// </summary>
        /// <param name="drawingContext">드로잉 컨텍스트</param>
        /// <param name="stylusPointCollection">스타일러스 포인트 컬렉션</param>
        /// <param name="geometry">지오메트리</param>
        /// <param name="fillBrush">채우기 브러시</param>
        protected override void OnDraw(DrawingContext drawingContext, StylusPointCollection stylusPointCollection, Geometry geometry, Brush fillBrush)
        {
            _brush = _brush ?? new LinearGradientBrush(Colors.Red, Colors.Blue, 20d);

            _pen = _pen ?? new Pen(_brush, 2d);

            for(int i = 0; i < stylusPointCollection.Count; i++)
            {
                Point  point = (Point)stylusPointCollection[i];

                Vector vector  = Point.Subtract(this.previousPoint, point);

                if(vector.Length > 4)
                {
                    double radius = stylusPointCollection[i].PressureFactor * 10d;

                    drawingContext.DrawEllipse(_brush, _pen, point, radius, radius);

                    this.previousPoint = point;
                }
            }
        }

        #endregion
    }
}