﻿using System.Windows;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 커스텀 스트로크
    /// </summary>
    public class CustomStroke : Stroke
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 브러시
        /// </summary>
        private Brush brush;

        /// <summary>
        /// 펜
        /// </summary>
        private Pen pen;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomStroke(stylusPointCollection)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="stylusPointCollection">스타일러스 포인트 컬렉션</param>
        public CustomStroke(StylusPointCollection stylusPointCollection) : base(stylusPointCollection)
        {
            this.brush = new LinearGradientBrush(Colors.Red, Colors.Blue, 20d);
            this.pen   = new Pen(this.brush, 2d);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 그리기 (코어) - DrawCore(drawingContext, drawingAttributes)

        /// <summary>
        /// 그리기 (코어)
        /// </summary>
        /// <param name="drawingContext">드로잉 컨텍스트</param>
        /// <param name="drawingAttributes">드로잉 어트리뷰트</param>
        protected override void DrawCore(DrawingContext drawingContext, DrawingAttributes drawingAttributes)
        {
            Point previousPoint = new Point(double.NegativeInfinity, double.NegativeInfinity);

            for(int i = 0; i < StylusPoints.Count; i++)
            {
                Point  point  = (Point)StylusPoints[i];
                Vector vector = Point.Subtract(previousPoint, point);

                if(vector.Length > 4)
                {
                    double radius = StylusPoints[i].PressureFactor * 10d;

                    drawingContext.DrawEllipse(this.brush, this.pen, point, radius, radius);

                    previousPoint = point;
                }
            }
        }

        #endregion
    }
}